/*
 * TasksFrame.java
 *
 * Created on 09 May 2006, 09:33
 */

package com.builder.uk.todotasks;

import java.util.Date;
import javax.swing.DefaultCellEditor;
import javax.swing.JComboBox;
import javax.swing.ListSelectionModel;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.table.TableColumn;

/**
 *
 * @author  Dj
 */
public class TasksFrame extends javax.swing.JFrame implements ListSelectionListener {
    Controller controller;
    
    TasksModel tasksModel;
    TableSorter sorter;
    
    /**
     * Creates new form TasksFrame
     */
    public TasksFrame(Controller controller) {
        this.controller=controller;
        
        initComponents();
        
        tasksModel=controller.getTasksModel();
        sorter=new TableSorter(tasksModel);
        tasksTable.setModel(sorter);
        sorter.setTableHeader(tasksTable.getTableHeader());
    
        tasksTable.getSelectionModel().setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        tasksTable.getSelectionModel().addListSelectionListener(this);
        
       
        DefaultCellEditor priorityEditor=new DefaultCellEditor(new JComboBox(Priority.values()));
        priorityEditor.setClickCountToStart(2);
        tasksTable.setDefaultEditor(Priority.class,priorityEditor);
        
        tasksTable.setDefaultRenderer(Date.class,new DateCellRenderer());
        tasksTable.setDefaultEditor(Date.class,new DateCellEditor());
        
        TableColumn doneColumn=tasksTable.getColumnModel().getColumn(3);
        doneColumn.setCellEditor(new DoneCellEditor());
        doneColumn.setCellRenderer(new DoneCellRenderer());
        doneColumn.setPreferredWidth(50);
        
        tasksTable.getColumnModel().getColumn(0).setMaxWidth(40);
        tasksTable.getColumnModel().getColumn(1).setPreferredWidth(200);
        tasksTable.getColumnModel().getColumn(2).setMaxWidth(80);
    }
    
    
    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    // <editor-fold defaultstate="collapsed" desc=" Generated Code ">//GEN-BEGIN:initComponents
    private void initComponents() {
        jScrollPane1 = new javax.swing.JScrollPane();
        tasksTable = new javax.swing.JTable();
        jScrollPane2 = new javax.swing.JScrollPane();
        notesTextArea = new javax.swing.JTextArea();
        createButton = new javax.swing.JButton();
        deleteButton = new javax.swing.JButton();
        tasksMenuBar = new javax.swing.JMenuBar();
        fileMenu = new javax.swing.JMenu();
        quitMenuItem = new javax.swing.JMenuItem();

        setDefaultCloseOperation(javax.swing.WindowConstants.EXIT_ON_CLOSE);
        setTitle("ToDo Tasks");
        addWindowListener(new java.awt.event.WindowAdapter() {
            public void windowClosed(java.awt.event.WindowEvent evt) {
                formWindowClosed(evt);
            }
        });

        tasksTable.setShowHorizontalLines(false);
        jScrollPane1.setViewportView(tasksTable);

        notesTextArea.setColumns(20);
        notesTextArea.setRows(5);
        jScrollPane2.setViewportView(notesTextArea);

        createButton.setText("+");
        createButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                createButtonActionPerformed(evt);
            }
        });

        deleteButton.setText("-");
        deleteButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                deleteButtonActionPerformed(evt);
            }
        });

        fileMenu.setText("File");
        quitMenuItem.setText("Quit");
        quitMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                quitMenuItemActionPerformed(evt);
            }
        });

        fileMenu.add(quitMenuItem);

        tasksMenuBar.add(fileMenu);

        setJMenuBar(tasksMenuBar);

        org.jdesktop.layout.GroupLayout layout = new org.jdesktop.layout.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(layout.createSequentialGroup()
                .addContainerGap()
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(jScrollPane2, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 579, Short.MAX_VALUE)
                    .add(layout.createSequentialGroup()
                        .add(createButton)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(deleteButton))
                    .add(jScrollPane1, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 579, Short.MAX_VALUE))
                .addContainerGap())
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(org.jdesktop.layout.GroupLayout.TRAILING, layout.createSequentialGroup()
                .addContainerGap()
                .add(jScrollPane1, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 138, Short.MAX_VALUE)
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(jScrollPane2, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 136, Short.MAX_VALUE)
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(createButton)
                    .add(deleteButton))
                .addContainerGap())
        );
        pack();
    }// </editor-fold>//GEN-END:initComponents

    private void quitMenuItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_quitMenuItemActionPerformed
        controller.quit();
    }//GEN-LAST:event_quitMenuItemActionPerformed

    private void formWindowClosed(java.awt.event.WindowEvent evt) {//GEN-FIRST:event_formWindowClosed
         controller.quit();
    }//GEN-LAST:event_formWindowClosed
    
    private void deleteButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_deleteButtonActionPerformed
        if(tasksTable.getSelectedRow()==-1) return;
        
        Long id=(Long)sorter.getValueAt(tasksTable.getSelectedRow(),0);
        
        controller.removeTask(id);
    }//GEN-LAST:event_deleteButtonActionPerformed
    
    private void createButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_createButtonActionPerformed
        controller.createTask();
    }//GEN-LAST:event_createButtonActionPerformed
    

    public void valueChanged(ListSelectionEvent e) {
        int row=tasksTable.getSelectedRow();
        
        if(row==-1)
        {
            controller.taskSelected(null);
            return;
        }
        
        Long id=(Long)sorter.getValueAt(tasksTable.getSelectedRow(),0);
        controller.taskSelected(id);
    }

    String getNotes() {
        return notesTextArea.getText();
    }
    
    void setNotes(String notes) {
        notesTextArea.setText(notes);
    }

    void setNotesEnabled(boolean enable)
    {
        notesTextArea.setEnabled(enable);
    }
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    javax.swing.JButton createButton;
    javax.swing.JButton deleteButton;
    javax.swing.JMenu fileMenu;
    javax.swing.JScrollPane jScrollPane1;
    javax.swing.JScrollPane jScrollPane2;
    javax.swing.JTextArea notesTextArea;
    javax.swing.JMenuItem quitMenuItem;
    javax.swing.JMenuBar tasksMenuBar;
    javax.swing.JTable tasksTable;
    // End of variables declaration//GEN-END:variables
    
}
