/*
 * TasksSession.java
 */

package com.builder.uk.todotasks;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.Date;
import java.util.List;

public class TasksSession implements TasksListener {
    
    // Static controller field, initialised by the controller at startup.
    private static Controller controller;
    
    public static void setController(Controller theController) {
        controller=theController;
    }
    
    Tasks tasks;
    
    SortOrder sortingfield=new SortOrder(SortOrder.TaskField.TASKID,false);
    
    ArrayList<Task> tasklist;
    
    public TasksSession() throws IOException {
        System.out.println("Created TaskSession");
        tasks=controller.getTasks();
        tasks.addTasksListener(this);
        tasklist=new ArrayList<Task>();
        List<Long> taskids=tasks.getTaskIds();
        
        for(Long l:taskids) {
            tasklist.add(tasks.getTask(l));
        }
        
        Collections.sort(tasklist,new TaskComparator());
    }
    
  
    
    public Task getTask(Long taskId) {
        for(Task t:tasklist) {
            if(t.getTaskId().equals(taskId)) return t;
        }
        
        return null;
    }
    
    public Task createTask() throws IOException {
        return tasks.createTask();
    }
    
    public boolean deleteTask(Long taskId) {
        try {
            tasks.removeTask(taskId);
        } catch (IOException ex) {
            return false;
        }
        
        return true;
    }
    
    public boolean updateTask(Task task) {
        try {
            tasks.update(task);
        } catch (IOException ex) {
            return false;
        }
        return true;
    }
    
   
    
    public Task[] getAllTasks() {
        return (Task [])tasklist.toArray(new Task[tasklist.size()]);
    }
    
    public void setSortOrder(SortOrder tf) {
        sortingfield=tf;
        Collections.sort(tasklist,new TaskComparator());
    }
    
    public SortOrder getSortOrder() { return sortingfield; }
    
    public void tasksChanged(TasksEvent tasksevent) {
        switch(tasksevent.taskEventType) {
            
            case CHANGED:
                for(Task t:tasklist) {
                    if(t.getTaskId().equals(tasksevent.taskId)) {
                        tasklist.remove(t);
                        break;
                    }
                }
                // Deliberate fall through
            case NEW:
                
                try {
                    tasklist.add(tasks.getTask(tasksevent.taskId));
                    Collections.sort(tasklist,new TaskComparator());
                } catch (IOException ex) {
                    ex.printStackTrace();
                }
                
                break;
                
            case DELETED:
                for(Task t:tasklist) {
                    if(t.getTaskId().equals(tasksevent.taskId)) {
                        tasklist.remove(t);
                        break;
                    }
                }
                break;
        }
        
    }
    
      private class TaskComparator implements Comparator {
        public int compare(Object o1, Object o2) {
            Task t1=(Task)o1;
            Task t2=(Task)o2;
            int result=0;
            
            switch(sortingfield.getField()) {
                case TASKID:
                    result= t1.getTaskId().compareTo(t2.getTaskId());
                    break;
                case NAME:
                    result= t1.getName().compareTo(t2.getName());
                    break;
                case PRIORITY:
                    result= t1.getPriority().compareTo(t2.getPriority());
                    break;
                case DONE:
                    result= ((Integer)t1.getDonePercent()).compareTo(t2.getDonePercent());
                    break;
                case COMPLETED:
                    Date d1=t1.getCompletedDate();
                    Date d2=t2.getCompletedDate();
                    if(d1==null && d2==null) { result= 0; break; }
                    if(d1==null) { result=-1; break; }
                    if(d2==null) { result=1; break; };
                    result=d1.before(d2) ? -1 : 1;
                    break;
                case DUE:
                    Date dd1=t1.getDueDate();
                    Date dd2=t2.getDueDate();
                    if(dd1==null && dd2==null) {result=0; break; }
                    if(dd1==null) { result=-1; break; }
                    if(dd2==null) { result=1; break; }
                    result=dd1.before(dd2) ? -1 : 1;
                    break;
            }
            
            if(sortingfield.isDescending()) return -result;
            return result;
        }
        
    }
    
    
}
